{=======================================================================================================================

  Comming Library.

     .

========================================================================================================================

  :

  25.03.2019 [!]   D7  XE.
  05.05.2005 [+] SumTaxStr:       .
  26.04.2004 [*] StrToCurrency:   .
  16.07.2004 [+] IntToAlpha:     "a".
  24.03.2004 [+] RoundFor:    .
  05.06.2003 [+]      ,   ().
  19.02.2003 [+]    (   ).
  10.10.2001 [-]  TAB.
  09.09.1998 [!]  Comming Library.

========================================================================================================================
  Copyright 19982019 TSK Developer Laboratory, Moscow.                                         Konstantin Tsvetkov
                                                                                                  e-mail: dev@tsklab.ru
                                                                                                  http://www.tsklab.ru
  CVS: $Id: CommingMath.pas,v 1.3 2019/03/25 14:02:27 tsk Exp $
=======================================================================================================================}
unit CommingMath;

interface

  uses SysUtils, Math, CommingString;

  function  MaximaInt( One, Two: Integer ): Integer;
  function  xExp ( n, i: Integer ): Integer;
  function  EncodeHash( n, f, b, l: Integer ): String;
  function  DecodeHash( Code: String; f, b: Integer ): Integer;
  function  RoundMoney( X: Extended ): Currency;
  //       .
{$IFDEF VER150} // D7
  function  StrToCurrency( Value: String ): Currency;
  function  SumTax( Quantity, Price, Tax, TaxCash: Double ): Currency; overload;
  function  SumTax( Quantity, Price, Tax, TaxCash: String ): Currency; overload;
  function  SumTaxStr( Quantity, Price, Tax, TaxCash: String ): String;
{$ELSE} // XE
  function  SumTax( Quantity, Price, Tax, TaxCash: Double ): Currency;
{$ENDIF}
  //    .
  function  RoundFor( Value, Delta: Extended): Extended;
  //     "a".
  function  IntToAlpha( Value: Integer; Digits: Integer ): String;

implementation
//======================================================================================================================
function MaximaInt( One, Two: Integer ): Integer ;
begin
  Result := One;
  if Two > One then Result := Two
end;
//======================================================================================================================
function xExp ( n, i: Integer ): Integer;
begin
  Result := 1;
  while i > 0 do begin Result := Result * n; Dec( i ) end;
end;
//======================================================================================================================
function EncodeHash( n, f, b, l: Integer ): String;
  var i, d, a: Integer;
begin
  i := 1;
  while n >= xExp( b, i ) do i := i + 1;
  i := i - 1;
  Result := '';
  while i >= 0 do
    begin
      a := xExp( b, i );
      d := Round( Int( n / a ));
      n := n - d * a;
      Result := Result + Chr( f + d );
      Dec( i );
    end;
  while Length( Result ) < l do Result := Chr( f ) + Result;
end;
//======================================================================================================================
function DecodeHash( Code: String; f, b: Integer ): Integer;
  var l, m, i: Integer;
begin
  Result := 0;
  l := Length( Code );
  m := xExp( b, l - 1 );
  while Length( Code ) > 0 do
    begin
      i := Ord( Code[1] );
      Result := Result + (( i - f ) * m );
      Code := Copy( Code, 2, l );
      m := Round( Int( m / b ));
    end;
end;
//======================================================================================================================
function RoundMoney( X: Extended ): Currency;
begin
  Result := RoundTo( X, -2 );// * 100 ) / 100;
end;
//======================================================================================================================
function SumTax( Quantity, Price, Tax, TaxCash: Double ): Currency;
begin
  Result := RoundTo( Quantity * Price, -2 );
  Result := Result + RoundTo( Result * Tax, -2 );
  Result := Result + RoundTo( Result * TaxCash, -2 );
end;
//======================================================================================================================
{$IFDEF VER150} // D7
function  StrToCurrency( Value: String ): Currency;
  var p: Integer;
begin
  p := Pos( CurrencyString, Value );
  if ( p > 0 ) then Value := Copy( Value, 1, p - 1);
  Value := DeleteChars( Value, #32#160 );
  if not TextToFloat( PChar( Value ), Result, fvCurrency ) then Result := 0;
end;
//======================================================================================================================
function SumTax( Quantity, Price, Tax, TaxCash: String ): Currency;
begin
  Result := RoundTo( StrToFloatDef( Quantity, 0 ) * StrToCurrency( Price ), -2 );
  Result := Result + RoundTo( Result * StrToFloatDef( Tax, 0 ) / 100, -2 );
  Result := Result + RoundTo( Result * StrToFloatDef( TaxCash, 0 ) / 100, -2 );
end;
//======================================================================================================================
function SumTaxStr( Quantity, Price, Tax, TaxCash: String ): String;
begin
  Result := CurrencyToStr( SumTax( Quantity, Price, Tax, TaxCash ));
end;
{$ENDIF}
//======================================================================================================================
function RoundFor( Value, Delta: Extended): Extended;
begin
  Result := Round ( Value / Delta ) * Delta;
end;
//======================================================================================================================
function IntToAlpha( Value: Integer; Digits: Integer ): String;
begin
  if Digits > 0 then Result := IntToAlpha(( Value div 26 ), Digits - 1 ) + Chr((Value mod 26) + 97 );
end;
//======================================================================================================================
end.
