{=======================================================================================================================

  Comming Library.

        .

========================================================================================================================

  :

  17.09.2014 [!]   D7  XE.
  16.09.2014 [!]  Comming D7  XE.
  09.02.2012 [!]    Delphi XE2.
  14.11.2006 [*]    -.
  24.03.2004 [*]    .
  14.11.2002 [*]  TAB.
  16.07.2002 [+]    null.
             [*]      AS.
  16.01.2002 [*]       .
             [+]  .
             [+]   .
  06.12.2001 [+]     , .
  14.11.2001 [+]    .
  13.11.2001 [+]   .
             [+]   .
  21.08.2000 [+]      TDate  String.
                  .
  09.09.1998 [!]  Comming Library.

========================================================================================================================
  Copyright 19982014 TSK Developer Laboratory, Moscow.                                         Konstantin Tsvetkov
                                                                              e-mail: dev@tsklab.ru
                                                                              http://www.tsklab.ru
  CVS: $Id: CommingRegistry.pas,v 1.2 2017/08/26 08:41:50 tsk Exp $
=======================================================================================================================}
unit CommingRegistry;

interface

uses Windows, Classes, SysUtils, Variants, Forms, DB,
     {$IFDEF VER150} // D7
     Registry;
     {$ELSE} // XE
     System.Win.Registry;
     {$ENDIF}

  function  RegistryGetValue( aRoot: HKey; const Key, ValueName: String; DefaultValue: Variant ): Variant;
  procedure RegistrySetValue( aRoot: HKey; const Key, ValueName: String; Value: Variant );

  function  RestoreDefaultValue( ValueName: String; DefaultValue: Variant; Path: String ): Variant; overload;
  function  RestoreDefaultValue( ValueName: String; DefaultValue: Variant ): Variant; overload;
  procedure RestoreDefaultValue( Field: TField; DefaultValue: Variant ); overload;

  procedure SaveDefaultValue( ValueName: String; DefaultValue: Variant; Path: String ); overload;
  procedure SaveDefaultValue( ValueName: String; DefaultValue: Variant ); overload;
  procedure SaveDefaultValue( Field: TField ); overload;

  const FieldNull = '<null>';

implementation

//======================================================================================================================
function  FormName( Name: String ): String;
begin
  Result := Copy( Name, 1, Length( Name ) - 2 );
end;
//======================================================================================================================
function RegistryGetValue( aRoot: HKey; const Key, ValueName: String; DefaultValue: Variant ): Variant;
  var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  with Reg do begin
    RootKey := aRoot;
    if OpenKeyReadOnly( Key ) and ValueExists( ValueName )
      then case VarType( DefaultValue )of
          varInteger, varShortInt, varByte, varInt64, varWord:
            Result := ReadInteger( ValueName );
          varDouble:
            Result := ReadFloat( ValueName );
          varCurrency:
            Result := ReadCurrency( ValueName );
          varDate:
            Result := ReadDate( ValueName );
          varString {$IFNDEF VER150}, varUString {$ENDIF}:
            Result := ReadString( ValueName );
          else raise Exception.CreateFmt( '  (%d)  .',
                                          [ VarType( DefaultValue )]);
        end
      else Result := DefaultValue;
    Free;
  end;
end;
//======================================================================================================================
procedure RegistrySetValue( aRoot: HKey; const Key, ValueName: String; Value: Variant );
  var Reg: TRegistry;
begin
  Reg := TRegistry.Create;
  with Reg do begin
    RootKey := aRoot;
    if ( OpenKey( Key, True )) then begin
      case VarType( Value )of
        varNull:
          WriteInteger( ValueName, 0 );
        varInteger, varShortInt, varByte:
          WriteInteger( ValueName, Value );
        varDouble:
          WriteFloat( ValueName, Value );
        varCurrency:
          WriteCurrency( ValueName, Value );
        varDate:
          WriteDate( ValueName, Value );
          varString {$IFNDEF VER150}, varUString {$ENDIF}:
          WriteString( ValueName, Value );
        else raise Exception.CreateFmt( '  (%d)  .',[ VarType( Value )]);
      end;
      CloseKey;
    end;
    Free;
  end;
end;
//======================================================================================================================
function RestoreDefaultValue( ValueName: String; DefaultValue: Variant; Path: String ): Variant;
begin
  Result := RegistryGetValue( HKEY_CURRENT_USER, Path,
                              ValueName + '.DefaultValue', DefaultValue )
end;
//======================================================================================================================
function  RestoreDefaultValue( ValueName: String; DefaultValue: Variant ): Variant;
begin
  Result := RegistryGetValue( HKEY_CURRENT_USER,
                              'Software\TSK Developer Laboratory\' + Application.Title + '\DefaultValue',
                              ValueName, DefaultValue )
end;
//======================================================================================================================
procedure RestoreDefaultValue( Field: TField; DefaultValue: Variant ); overload;
  var Reg: TRegistry;
      Key: String;
begin
  if not ( Field.DataSet.State = dsInsert ) then Exit;
  Reg := TRegistry.Create;
  Key := 'Software\TSK Developer Laboratory\' +
         Application.Title + '\' + FormName( Field.Owner.Name ) +
         '\DefaultValue';
  with Reg do begin
    RootKey := HKEY_CURRENT_USER;
    if OpenKey( Key, False ) and ValueExists( Field.FieldName ) then
      if ( GetDataType( Field.FieldName ) = rdString ) and ( ReadString( Field.FieldName ) = FieldNull )
        then Field.Clear
        else
          case Field.DataType of
            ftInteger, ftSmallint:
              Field.Value := ReadInteger( Field.FieldName );
            ftFloat:
              Field.Value := ReadFloat( Field.FieldName );
            ftDateTime, ftDate:
              Field.Value := ReadDateTime( Field.FieldName );
            ftString:
              Field.Value := ReadString( Field.FieldName );
            ftCurrency:
              Field.Value := ReadCurrency( Field.FieldName )
            else raise Exception.CreateFmt( '  (%d)   %s.',
                                          [ Ord( Field.DataType ), Field.FieldName ]);
        end
    else Field.Value := DefaultValue;
    CloseKey;
    Free;
  end;
end;
//======================================================================================================================
procedure SaveDefaultValue( ValueName: String; DefaultValue: Variant; Path: String );
begin
  RegistrySetValue( HKEY_CURRENT_USER, Path,
                    ValueName + '.DefaultValue', DefaultValue )
end;
//======================================================================================================================
procedure SaveDefaultValue( ValueName: String; DefaultValue: Variant );
begin
  RegistrySetValue( HKEY_CURRENT_USER,
                    'Software\TSK Developer Laboratory\' + Application.Title + '\DefaultValue',
                    ValueName, DefaultValue )
end;
//======================================================================================================================
procedure SaveDefaultValue( Field: TField );
  var Reg: TRegistry;
      Key: String;
begin
  Reg := TRegistry.Create;
  Key := 'Software\TSK Developer Laboratory\' +
         Application.Title + '\' + FormName( Field.Owner.Name ) +
         '\DefaultValue';
  with Reg do begin
    RootKey := HKEY_CURRENT_USER;
    if ( KeyExists( Key ) or CreateKey( Key )) and ( OpenKey( Key, False )) then begin
      if Field.IsNull
        then WriteString( Field.FieldName, FieldNull )
        else
          case Field.DataType of
            ftInteger, ftSmallint:
              WriteInteger( Field.FieldName, Field.AsInteger );
            ftFloat:
              WriteFloat( Field.FieldName, Field.AsFloat );
            ftDateTime, ftDate:
              WriteDateTime(Field.FieldName, Field.AsDateTime );
            ftString:
              WriteString( Field.FieldName, Field.AsString );
            ftCurrency:
              WriteCurrency( Field.FieldName, Field.AsCurrency )
            else raise Exception.CreateFmt( '  (%d)   %s.',
                                          [ Ord( Field.DataType ), Field.FieldName ]);
          end;
      CloseKey;
    end;
    Free;
  end;
end;
//======================================================================================================================
end.


