{=======================================================================================================================

  Comming Library.

     .

========================================================================================================================

  :

  19.06.2016 [+] IsWindows10.
  13.12.2015 [+] KeyDownShift, KeyDownCtrl, KeyDownAlt:  .
  18.04.2015 [-] ClipboardText.
  09.04.2015 [*] ClipboardText:    Delphi 7.
  16.09.2014 [!]  Comming D7  XE
  30.01.2014 [+] ClipboardText:    .
  09.03.2013 [*] .
  15.06.2010 [+] IsVista:    5.
  22.05.2008 [-]      CommingFile.
  14.12.2004 [+] StopProcess:     .
  24.03.2004 [*] .
  11.03.2001 [+]  " ".
             [+]    (  ).
  24.01.2001 [+] GetCommandLineNumb -      .
  16.12.2000 [-]   GetBuild. .  CommingVersionInfo.
  04.09.2000 [*]      .
  09.04.1999 [+]  .
  09.09.1998 [!]  Comming Library.

========================================================================================================================
  Copyright 19982016 TSK Developer Laboratory, Moscow.                                         Konstantin Tsvetkov
                                                                                                  e-mail: dev@tsklab.ru
                                                                                                  http://www.tsklab.ru
  CVS: $Id: CommingSystem.pas,v 1.6 2016/12/14 15:57:51 tsk Exp $
=======================================================================================================================}
unit CommingSystem;

interface

  uses SysUtils, Windows, Dialogs, ShellAPI, TlHelp32, Forms, CommingRegistry, CommingFile;

type
  TExecState = ( esNormal, esMinimized, esMaximized, esHidden );

  function  GetCompNo: String;
  function  GetUserNickName: String;
  function  GetCompName: String;
  function  GetRegisteredOwner: String;
  function  GetRegisteredOrganization: String;
  function  GetCommandLineNumb( Param: String ): Integer;
  function  FileExecute(const FileName, Params, StartDir: string; InitialState: TExecState): THandle;
  procedure StopProcess( NameProcess: String );
  function  IsVista: Boolean;
  function  IsWindows10: Boolean;
  //  :
  function  KeyDownShift: Boolean;
  function  KeyDownCtrl: Boolean;
  function  KeyDownAlt: Boolean;

const ShowCommands: array[TExecState] of Integer = (SW_SHOWNORMAL, SW_MINIMIZE, SW_SHOWMAXIMIZED, SW_HIDE);

implementation
//======================================================================================================================
function GetCompNo: String ;
  var N: Integer;
begin
  N := GetVolumeNo( 'C' );
  Result := IntToHex( HiWord( N ), 4 ) + '-' + IntToHex( LoWord( N ), 4 );
end;
//======================================================================================================================
function GetUserNickname: String;
  var NameBuffer: PChar;
      SizeBuffer: DWORD;
begin
  SizeBuffer := 40;
  NameBuffer := StrAlloc( SizeBuffer + 1 );
  try
    GetUserName( NameBuffer, SizeBuffer );
    Result := NameBuffer;
  finally
    StrDispose( NameBuffer );
  end;
end;
//======================================================================================================================
function  GetCompName: String;
  var NameBuffer: PChar;
      SizeBuffer: DWORD;
begin
  SizeBuffer := 100;
  NameBuffer := StrAlloc( SizeBuffer + 1 );
  try
    GetComputerName( NameBuffer, SizeBuffer );
    Result := NameBuffer;
  finally
    StrDispose( NameBuffer );
  end;
end;
//======================================================================================================================
function  GetRegisteredOwner: String;
begin
  Result := RegistryGetValue( HKEY_LOCAL_MACHINE,
                              '\SOFTWARE\Microsoft\Windows\CurrentVersion',
                              'RegisteredOwner',
                              '*' );
  if Result = '*' then
    Result := RegistryGetValue( HKEY_LOCAL_MACHINE,
                              '\SOFTWARE\Microsoft\Windows NT\CurrentVersion',
                              'RegisteredOwner',
                              '*' );
end;
//======================================================================================================================
function  GetRegisteredOrganization: String;
begin
  Result := RegistryGetValue( HKEY_LOCAL_MACHINE,
                              '\SOFTWARE\Microsoft\Windows\CurrentVersion',
                              'RegisteredOrganization',
                              '*' );
  if Result = '*' then
    Result := RegistryGetValue( HKEY_LOCAL_MACHINE,
                              '\SOFTWARE\Microsoft\Windows NT\CurrentVersion',
                              'RegisteredOrganization',
                              '*' );
end;
//======================================================================================================================
function FileExecute(const FileName, Params, StartDir: string; InitialState: TExecState): THandle;
begin
  Result := ShellExecute( Application.Handle, nil, PChar(FileName), PChar(Params), PChar(StartDir),
                          ShowCommands[ InitialState ]);
end;
//======================================================================================================================
function GetCommandLineNumb( Param: String ): Integer;
  var sComm: String;
      iPos: Integer;
begin
  Result  := 0;
  sComm   := AnsiUpperCase( GetCommandLine );
  iPos    := Pos( AnsiUpperCase( Param ), sComm );
  if iPos > 0 then begin
    sComm := Copy( sComm, iPos + Length( Param ) + 1, Length( sComm ));
    iPos := Pos( ' ', sComm );
    if iPos > 0 then sComm := Copy( sComm, 1, iPos );
    Result := StrToIntDef( sComm, 0 );
  end;
end;
//======================================================================================================================
procedure StopProcess( NameProcess: String );
  var SnapshotHandle, ProcessHandle: THandle;
      ProcessEntry32: TProcessEntry32;
begin
  NameProcess := AnsiUpperCase( NameProcess );
  SnapshotHandle := CreateToolhelp32Snapshot( TH32CS_SNAPPROCESS, 0);
  ProcessEntry32.dwSize := Sizeof( ProcessEntry32 );
  Process32First( SnapshotHandle, ProcessEntry32);
  repeat
    if AnsiUpperCase( ProcessEntry32.szExeFile ) = NameProcess then begin
      ProcessHandle := OpenProcess( PROCESS_TERMINATE, BOOL(0), ProcessEntry32.th32ProcessID );
      if ProcessHandle <> 0 then begin
        TerminateProcess( ProcessHandle, 0 );
        CloseHandle(ProcessHandle);
      end;
    end;
  until not Process32Next( SnapshotHandle, ProcessEntry32);
  CloseHandle( SnapshotHandle );
end;
//======================================================================================================================
function  IsVista: Boolean;
  var OSVersionInfo : TOSVersionInfo;
begin
  OSVersionInfo.dwOSVersionInfoSize := sizeof(TOSVersionInfo);
  GetVersionEx( OSVersionInfo );
  Result := OSVersionInfo.dwMajorVersion > 5;
end;
//======================================================================================================================
function  IsWindows10: Boolean;
  var OSVersionInfo : TOSVersionInfo;
begin
  OSVersionInfo.dwOSVersionInfoSize := sizeof(TOSVersionInfo);
  GetVersionEx( OSVersionInfo );
  Result := ((OSVersionInfo.dwMajorVersion) >= 6) and (OSVersionInfo.dwMinorVersion > 1);
//  ShowMessageFmt( '%d', [ OSVersionInfo.dwMinorVersion ]);
end;
//======================================================================================================================
function  KeyDownShift: Boolean;
begin
  Result := ( Word( GetKeyState( VK_SHIFT )) and $8000 ) <> 0;
end;
//======================================================================================================================
function  KeyDownCtrl: Boolean;
begin
  Result := ( Word( GetKeyState( VK_CONTROL )) and $8000 ) <> 0;
end;
//======================================================================================================================
function  KeyDownAlt: Boolean;
begin
  Result := ( Word( GetKeyState( VK_MENU )) and $8000 ) <> 0;
end;
//======================================================================================================================
end.
